@@ -1,20 +1,19 @@
-/*
-*   DSP API - a digital signal processing library for astronomy usage
-*   Copyright © 2017-2022  Ilia Platone
+/*  libDSP - a digital signal processing library
+*   Copyright © 2017-2023  Ilia Platone
 *
-*   This program is free software; you can redistribute it and/or
-*   modify it under the terms of the GNU Lesser General Public
-*   License as published by the Free Software Foundation; either
-*   version 3 of the License, or (at your option) any later version.
+*   This program is free software; you can redistribute it and/or modify
+*   it under the terms of the GNU General Public License as published by
+*   the Free Software Foundation; either version 2 of the License, or
+*   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-*   Lesser General Public License for more details.
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*   GNU General Public License for more details.
 *
-*   You should have received a copy of the GNU Lesser General Public License
-*   along with this program; if not, write to the Free Software Foundation,
-*   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+*   You should have received a copy of the GNU General Public License along
+*   with this program; if not, write to the Free Software Foundation, Inc.,
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 #include "dsp.h"
@@ -115,6 +114,10 @@ void dsp_stream_free_buffer(dsp_stream_p stream)
         free(stream->buf);
     if(stream->dft.buf != NULL)
         free(stream->dft.buf);
+    if(stream->magnitude != NULL)
+        dsp_stream_free_buffer(stream->magnitude);
+    if(stream->phase != NULL)
+        dsp_stream_free_buffer(stream->phase);
 }
 
 /**
@@ -180,6 +183,10 @@ void dsp_stream_free(dsp_stream_p stream)
         free(stream->stars);
     if(stream->triangles != NULL)
         free(stream->triangles);
+    if(stream->magnitude != NULL)
+        dsp_stream_free(stream->magnitude);
+    if(stream->phase != NULL)
+        dsp_stream_free(stream->phase);
     free(stream);
     stream = NULL;
 }
@@ -332,6 +339,9 @@ void dsp_stream_add_star(dsp_stream_p stream, dsp_star star)
     stream->stars = (dsp_star*)realloc(stream->stars, sizeof(dsp_star)*(stream->stars_count+1));
     strcpy(stream->stars[stream->stars_count].name, star.name);
     stream->stars[stream->stars_count].diameter = star.diameter;
+    stream->stars[stream->stars_count].peak = star.peak;
+    stream->stars[stream->stars_count].flux = star.flux;
+    stream->stars[stream->stars_count].theta = star.theta;
     stream->stars[stream->stars_count].center.dims = star.center.dims;
     stream->stars[stream->stars_count].center.location = (double*)malloc(sizeof(double)*star.center.dims);
     for(d = 0; d < star.center.dims; d++)
@@ -368,26 +378,35 @@ void dsp_stream_add_triangle(dsp_stream_p stream, dsp_triangle triangle)
 {
     int s;
     int d;
+    int num_baselines = triangle.stars_count*(triangle.stars_count-1)/2;
     stream->triangles = (dsp_triangle*)realloc(stream->triangles, sizeof(dsp_triangle)*(stream->triangles_count+1));
     stream->triangles[stream->triangles_count].dims = triangle.dims;
     stream->triangles[stream->triangles_count].index = triangle.index;
+    stream->triangles[stream->triangles_count].stars_count = triangle.stars_count;
     stream->triangles[stream->triangles_count].theta = (double*)malloc(sizeof(double)*(stream->dims-1));
-    stream->triangles[stream->triangles_count].ratios = (double*)malloc(sizeof(double)*triangle.dims);
-    stream->triangles[stream->triangles_count].sizes = (double*)malloc(sizeof(double)*triangle.dims);
-    stream->triangles[stream->triangles_count].stars = (dsp_star*)malloc(sizeof(dsp_star)*triangle.dims);
+    stream->triangles[stream->triangles_count].ratios = (double*)malloc(sizeof(double)*num_baselines);
+    stream->triangles[stream->triangles_count].sizes = (double*)malloc(sizeof(double)*num_baselines);
+    stream->triangles[stream->triangles_count].stars = (dsp_star*)malloc(sizeof(dsp_star)*triangle.stars_count);
     for (s = 0; s < triangle.dims; s++) {
         if(s < stream->dims - 1) {
             stream->triangles[stream->triangles_count].theta[s] = triangle.theta[s];
         }
-        stream->triangles[stream->triangles_count].sizes[s] = triangle.sizes[s];
-        stream->triangles[stream->triangles_count].ratios[s] = triangle.ratios[s];
+    }
+    for(s = 0; s < triangle.stars_count; s++) {
         stream->triangles[stream->triangles_count].stars[s].center.dims = triangle.stars[s].center.dims;
         stream->triangles[stream->triangles_count].stars[s].diameter = triangle.stars[s].diameter;
+        stream->triangles[stream->triangles_count].stars[s].peak = triangle.stars[s].peak;
+        stream->triangles[stream->triangles_count].stars[s].flux = triangle.stars[s].flux;
+        stream->triangles[stream->triangles_count].stars[s].theta = triangle.stars[s].theta;
         stream->triangles[stream->triangles_count].stars[s].center.location = (double*)malloc(sizeof(double)*stream->dims);
         for(d = 0; d < triangle.stars[s].center.dims; d++) {
             stream->triangles[stream->triangles_count].stars[s].center.location[d] = triangle.stars[s].center.location[d];
         }
     }
+    for(s = 0; s < num_baselines; s++) {
+        stream->triangles[stream->triangles_count].sizes[s] = triangle.sizes[s];
+        stream->triangles[stream->triangles_count].ratios[s] = triangle.ratios[s];
+    }
     stream->triangles_count++;
 }
 
@@ -398,16 +417,27 @@ void dsp_stream_add_triangle(dsp_stream_p stream, dsp_triangle triangle)
  */
 void dsp_stream_del_triangle(dsp_stream_p stream, int index)
 {
-    dsp_triangle* triangles = (dsp_triangle*)malloc(sizeof(dsp_triangle) * stream->triangles_count);
-    int triangles_count = stream->triangles_count;
-    memcpy(triangles, stream->triangles, sizeof(dsp_triangle*) * stream->triangles_count);
-    free(stream->triangles);
-    stream->triangles_count = 0;
+    int d;
+    free(stream->triangles[index].sizes);
+    free(stream->triangles[index].theta);
+    free(stream->triangles[index].ratios);
+    for(d = 0; d < stream->triangles[index].stars_count; d++) {
+        free(stream->triangles[index].stars[d].center.location);
+    }
+    free(stream->triangles[index].stars);
     int i;
-    for(i = 0; i < triangles_count; i++) {
-        if(i != index) {
-            dsp_stream_add_triangle(stream, triangles[i]);
+    for(i = index; i < stream->triangles_count - 1; i++) {
+        stream->triangles[i] = stream->triangles[i+1];
+    }
+    stream->triangles_count--;
+    if(index < stream->triangles_count) {
+        free(stream->triangles[i].sizes);
+        free(stream->triangles[i].theta);
+        free(stream->triangles[i].ratios);
+        for(d = 0; d < stream->triangles[i].dims; d++) {
+            free(stream->triangles[i].stars[d].center.location);
         }
+        free(stream->triangles[i].stars);
     }
 }
 
@@ -720,3 +750,119 @@ void dsp_stream_rotate(dsp_stream_p in)
     dsp_stream_free_buffer(stream);
     dsp_stream_free(stream);
 }
+
+static double stack_delegate_multiply(double x, double y)
+{
+    return sqrt(x * y);
+}
+
+static double stack_delegate_sum(double x, double y)
+{
+    return (x + y) / 2.0;
+}
+
+static double stack_delegate_subtraction(double x, double y)
+{
+    return fmax(0.0, x - y);
+}
+
+static void* dsp_stream_stack_th(void* arg)
+{
+    struct {
+       int cur_th;
+       dsp_stream_p stream;
+       double(*delegate)(double, double);
+    } *arguments = arg;
+    double(*delegate)(double, double) = arguments->delegate;
+    dsp_stream_p stream = arguments->stream;
+    dsp_stream_p in = stream->parent;
+    int cur_th = arguments->cur_th;
+    int start = cur_th * stream->len / dsp_max_threads(0);
+    int end = start + stream->len / dsp_max_threads(0);
+    end = Min(stream->len, end);
+    int y;
+    for(y = start; y < end; y++)
+    {
+        int *pos = dsp_stream_get_position(stream, y);
+        int x = dsp_stream_set_position(in, pos);
+        free(pos);
+        if(x >= 0 && x < in->len)
+            stream->buf[y] = delegate(stream->buf[y], in->buf[x]);
+    }
+    return NULL;
+}
+
+void dsp_stream_sum(dsp_stream_p in, dsp_stream_p str)
+{
+    dsp_stream_p stream = dsp_stream_copy(in);
+    stream->parent = str;
+    size_t y;
+    pthread_t *th = (pthread_t*)malloc(sizeof(pthread_t)*dsp_max_threads(0));
+    struct {
+       int cur_th;
+       dsp_stream_p stream;
+       double(*delegate)(double, double);
+    } thread_arguments[dsp_max_threads(0)];
+    for(y = 0; y < dsp_max_threads(0); y++) {
+        thread_arguments[y].cur_th = y;
+        thread_arguments[y].stream = stream;
+        thread_arguments[y].delegate = stack_delegate_sum;
+        pthread_create(&th[y], NULL, dsp_stream_stack_th, &thread_arguments[y]);
+    }
+    for(y = 0; y < dsp_max_threads(0); y++)
+        pthread_join(th[y], NULL);
+    free(th);
+    dsp_buffer_copy(stream->buf, in->buf, stream->len);
+    dsp_stream_free_buffer(stream);
+    dsp_stream_free(stream);
+}
+
+void dsp_stream_multiply(dsp_stream_p in, dsp_stream_p str)
+{
+    dsp_stream_p stream = dsp_stream_copy(in);
+    stream->parent = str;
+    size_t y;
+    pthread_t *th = (pthread_t*)malloc(sizeof(pthread_t)*dsp_max_threads(0));
+    struct {
+       int cur_th;
+       dsp_stream_p stream;
+       double(*delegate)(double, double);
+    } thread_arguments[dsp_max_threads(0)];
+    for(y = 0; y < dsp_max_threads(0); y++) {
+        thread_arguments[y].cur_th = y;
+        thread_arguments[y].stream = stream;
+        thread_arguments[y].delegate = stack_delegate_multiply;
+        pthread_create(&th[y], NULL, dsp_stream_stack_th, &thread_arguments[y]);
+    }
+    for(y = 0; y < dsp_max_threads(0); y++)
+        pthread_join(th[y], NULL);
+    free(th);
+    dsp_buffer_copy(stream->buf, in->buf, stream->len);
+    dsp_stream_free_buffer(stream);
+    dsp_stream_free(stream);
+}
+
+void dsp_stream_subtract(dsp_stream_p in, dsp_stream_p str)
+{
+    dsp_stream_p stream = dsp_stream_copy(in);
+    stream->parent = str;
+    size_t y;
+    pthread_t *th = (pthread_t*)malloc(sizeof(pthread_t)*dsp_max_threads(0));
+    struct {
+       int cur_th;
+       dsp_stream_p stream;
+       double(*delegate)(double, double);
+    } thread_arguments[dsp_max_threads(0)];
+    for(y = 0; y < dsp_max_threads(0); y++) {
+        thread_arguments[y].cur_th = y;
+        thread_arguments[y].stream = stream;
+        thread_arguments[y].delegate = stack_delegate_subtraction;
+        pthread_create(&th[y], NULL, dsp_stream_stack_th, &thread_arguments[y]);
+    }
+    for(y = 0; y < dsp_max_threads(0); y++)
+        pthread_join(th[y], NULL);
+    free(th);
+    dsp_buffer_copy(stream->buf, in->buf, stream->len);
+    dsp_stream_free_buffer(stream);
+    dsp_stream_free(stream);
+}
