@@ -55,6 +55,10 @@
 #ifndef MJS_CORE_PUBLIC_H_
 #define MJS_CORE_PUBLIC_H_
 
+#ifdef _WIN32
+#define _MSC_VER 1700
+#endif
+
 #if !defined(_MSC_VER) || _MSC_VER >= 1700
 #include <stdint.h>
 #else
