@@ -80,32 +80,40 @@ dsp_stream_p * vlbi_file_read_sdfits(char * filename, long *n)
         stream[r] = dsp_stream_new();
         int k = 0;
         dsp_fits_row row = rows[r];
-        for(k = 0; strcmp(SDFITS_COLUMN_OBJCTRA.name, row.columns[k].name); k++);
+        char * ra_str = SDFITS_COLUMN_OBJCTRA.name;
+        char * dec_str = SDFITS_COLUMN_OBJCTDEC.name;
+        char * freq_str = SDFITS_COLUMN_OBSFREQ.name;
+        char * lat_str = SDFITS_COLUMN_SITELAT.name;
+        char * lon_str = SDFITS_COLUMN_SITELONG.name;
+        char * el_str = SDFITS_COLUMN_SITEELEV.name;
+        char * date_str = SDFITS_COLUMN_DATE_OBS.name;
+        char * exposure_str = SDFITS_COLUMN_EXPOSURE.name;
+        char * time_str = SDFITS_COLUMN_TIME.name;
+        char * data_str = SDFITS_COLUMN_DATA.name;
+        for(k = 0; strcmp(ra_str, row.columns[k].name); k++);
         f_scansexa(row.columns[k].value, &stream[r]->target[0]);
-        for(k = 0; strcmp(SDFITS_COLUMN_OBJCTDEC.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(dec_str, row.columns[k].name); k++);
         f_scansexa(row.columns[k].value, &stream[r]->target[1]);
-        for(k = 0; strcmp(SDFITS_COLUMN_OBJCTDEC.name, row.columns[k].name); k++);
-        f_scansexa(row.columns[k].value, &stream[r]->target[1]);
-        for(k = 0; strcmp(SDFITS_COLUMN_OBSFREQ.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(freq_str, row.columns[k].name); k++);
         stream[r]->wavelength = *((double*)row.columns[k].value);
         stream[r]->wavelength = vlbi_astro_mean_speed(0) / stream[r]->wavelength;
-        for(k = 0; strcmp(SDFITS_COLUMN_SITELAT.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(lat_str, row.columns[k].name); k++);
         stream[r]->location[0].geographic.lat = *((double*)row.columns[k].value);
-        for(k = 0; strcmp(SDFITS_COLUMN_SITELONG.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(lon_str, row.columns[k].name); k++);
         stream[r]->location[0].geographic.lon = *((double*)row.columns[k].value);
-        for(k = 0; strcmp(SDFITS_COLUMN_SITEELEV.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(el_str, row.columns[k].name); k++);
         stream[r]->location[0].geographic.el = *((double*)row.columns[k].value);
-        for(k = 0; strcmp(SDFITS_COLUMN_DATE_OBS.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(date_str, row.columns[k].name); k++);
         stream[r]->starttimeutc = vlbi_time_string_to_timespec(row.columns[k].value);
-        for(k = 0; strcmp(SDFITS_COLUMN_EXPOSURE.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(exposure_str, row.columns[k].name); k++);
         stream[r]->samplerate = *((double*)row.columns[k].value);
-        for(k = 0; strcmp(SDFITS_COLUMN_TIME.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(time_str, row.columns[k].name); k++);
         double time = (long) * ((double*)row.columns[k].value);
         time -= stream[r]->samplerate / 2.0;
         stream[r]->starttimeutc.tv_sec += (long)time;
         stream[r]->starttimeutc.tv_nsec += (long)((time - stream[r]->starttimeutc.tv_sec) * 1000000000);
 
-        for(k = 0; strcmp(SDFITS_COLUMN_DATA.name, row.columns[k].name); k++);
+        for(k = 0; strcmp(data_str, row.columns[k].name); k++);
 
         dsp_fits_read_typecode(row.columns[k].format, &typecode, &width, &repeat);
         dsp_stream_add_dim(stream[r], (int)width);
