@@ -1,19 +1,19 @@
 /*  OpenVLBI - Open Source Very Long Baseline Interferometry
-*   Copyright © 2017-2023  Ilia Platone
+*   Copyright © 2017-2022  Ilia Platone
 *
-*   This program is free software; you can redistribute it and/or modify
-*   it under the terms of the GNU General Public License as published by
-*   the Free Software Foundation; either version 2 of the License, or
-*   (at your option) any later version.
+*   This program is free software; you can redistribute it and/or
+*   modify it under the terms of the GNU Lesser General Public
+*   License as published by the Free Software Foundation; either
+*   version 3 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*   GNU General Public License for more details.
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+*   Lesser General Public License for more details.
 *
-*   You should have received a copy of the GNU General Public License along
-*   with this program; if not, write to the Free Software Foundation, Inc.,
-*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*   You should have received a copy of the GNU Lesser General Public License
+*   along with this program; if not, write to the Free Software Foundation,
+*   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include <cstdio>
@@ -74,12 +74,10 @@ bool NodeCollection::contains(const char* element)
 void NodeCollection::setRelative(bool value)
 {
     relative = value;
-    if(value)
+    baselines->setRelative(value);
+    for(int x = 0; x < count(); x++)
     {
-        for(int x = 0; x < count(); x++)
-        {
-            memcpy(at(x)->stationLocation().coordinates, station.coordinates, sizeof(dsp_location));
-        }
+        memcpy(at(x)->stationLocation().coordinates, station.coordinates, sizeof(dsp_location));
     }
 }
 
