@@ -26,6 +26,7 @@
 #include <base64.h>
 #include <thread>
 
+NodeCollection *vlbi_nodes = new NodeCollection();
 static pthread_mutex_t mutex;
 static pthread_mutexattr_t mutexattr;
 static bool mutex_initialized = false;
@@ -67,8 +68,6 @@ static void unlock_mutex()
     pthread_mutex_unlock(&mutex);
 }
 
-static NodeCollection *vlbi_nodes = new NodeCollection();
-
 const char* vlbi_get_version()
 {
     return VLBI_VERSION_STRING;
