@@ -1,4 +1,22 @@
-﻿#include <cstdio>
+﻿/*  OpenVLBI - Open Source Very Long Baseline Interferometry
+*   Copyright © 2017-2023  Ilia Platone
+*
+*   This program is free software; you can redistribute it and/or modify
+*   it under the terms of the GNU General Public License as published by
+*   the Free Software Foundation; either version 2 of the License, or
+*   (at your option) any later version.
+*
+*   This program is distributed in the hope that it will be useful,
+*   but WITHOUT ANY WARRANTY; without even the implied warranty of
+*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*   GNU General Public License for more details.
+*
+*   You should have received a copy of the GNU General Public License along
+*   with this program; if not, write to the Free Software Foundation, Inc.,
+*   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*/
+
+#include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <signal.h>
@@ -14,7 +32,7 @@ static char *js_code_str;
 
 static double js_callback(double a, double b) {
     struct mjs *mjs = mjs_create();
-    char *str = (char*)malloc(strlen(js_code_str)+100);
+    char *str = (char*)malloc(strlen(js_code_str) + 100);
     sprintf(str, "function callback(a, b) { %s }; callback(%lf, %lf);", js_code_str, a, b);
     mjs_val_t result;
     mjs_exec(mjs, str, &result);
@@ -43,20 +61,26 @@ bool JSONServer::CheckMask(unsigned long mask, int n)
 
 void JSONServer::Parse()
 {
-    FILE* f = GetInput();
-    char *str = nullptr;
+    FILE* f = getInput();
     json_value *v;
     char *n;
     size_t len = 0;
-    int err = getdelim(&str, &len, (int)'\0', f);
-    (void)err;
+    char *str = (char*)malloc(1);
+    while(true) {
+        str[len] = 0;
+        fread(&str[len], 1, 1, f);
+        if(str[len++] == 0) break;
+        str = (char*)realloc(str, len + 8);
+    }
     if(strlen(str) <= 0)
         return;
+    if (len == 0)
+        return;
     char error[json_error_max];
     json_settings settings;
     memset(&settings, 0, sizeof(json_settings));
     json_value *value = json_parse_ex (&settings,
-                                       str, len-2,
+                                       str, len,
                                        error);
     if(!value)
     {
@@ -78,8 +102,8 @@ void JSONServer::Parse()
         json_values values = (json_values)v->u.object.values;
         if(!strcmp(n, "context"))
         {
-            AddContext(v->u.string.ptr);
-            SetContext(v->u.string.ptr);
+            addContext(v->u.string.ptr);
+            setContext(v->u.string.ptr);
         }
         if(!strcmp(n, "location"))
         {
@@ -106,7 +130,7 @@ void JSONServer::Parse()
             if(CheckMask(mask, 3))
             {
                 relative = true;
-                vlbi_set_location(GetContext(), lat, lon, el);
+                vlbi_set_location(getContext(), lat, lon, el);
             }
         }
         if(!strcmp(n, "fits"))
@@ -129,7 +153,7 @@ void JSONServer::Parse()
             }
             if(CheckMask(mask, 2))
             {
-                AddNode(name, base64);
+                addNode(name, base64);
             }
         }
         if(!strcmp(n, "sdfits"))
@@ -152,7 +176,7 @@ void JSONServer::Parse()
             }
             if(CheckMask(mask, 2))
             {
-                AddNodes(name, base64);
+                addNodes(name, base64);
             }
         }
         if(!strcmp(n, "node"))
@@ -178,7 +202,7 @@ void JSONServer::Parse()
                 }
                 if(!strcmp(values[y].name, "bitspersample"))
                 {
-                    SetBps(atoi(values[y].value->u.string.ptr));
+                    setBps(atoi(values[y].value->u.string.ptr));
                     mask |= 1 << 2;
                 }
                 if(!strcmp(values[y].name, "buffer"))
@@ -200,7 +224,7 @@ void JSONServer::Parse()
             }
             if(CheckMask(mask, 5))
             {
-                AddNode(name, (dsp_location*)locations, buf, buflen, starttime, relative);
+                addNode(name, (dsp_location*)locations, buf, buflen, starttime, relative);
             }
         }
         if(!strcmp(n, "plot"))
@@ -221,24 +245,24 @@ void JSONServer::Parse()
                     {
                         if(!strcmp(values[y].value->u.object.values[z].name, "ra"))
                         {
-                            SetRa(atof(values[y].value->u.object.values[z].value->u.string.ptr));
+                            setRa(atof(values[y].value->u.object.values[z].value->u.string.ptr));
                             mask |= 1 << 1;
                         }
                         if(!strcmp(values[y].value->u.object.values[z].name, "dec"))
                         {
-                            SetDec(atof(values[y].value->u.object.values[z].value->u.string.ptr));
+                            setDec(atof(values[y].value->u.object.values[z].value->u.string.ptr));
                             mask |= 1 << 2;
                         }
                     }
                 }
                 if(!strcmp(values[y].name, "frequency"))
                 {
-                    SetFreq(atof(values[y].value->u.string.ptr));
+                    setFreq(atof(values[y].value->u.string.ptr));
                     mask |= 1 << 3;
                 }
                 if(!strcmp(values[y].name, "samplerate"))
                 {
-                    SetSampleRate(atof(values[y].value->u.string.ptr));
+                    setSampleRate(atof(values[y].value->u.string.ptr));
                     mask |= 1 << 3;
                 }
                 if(!strcmp(values[y].name, "resolution"))
@@ -247,12 +271,12 @@ void JSONServer::Parse()
                     {
                         if(!strcmp(values[y].value->u.object.values[z].name, "width"))
                         {
-                            SetWidth(atoi(values[y].value->u.object.values[z].value->u.string.ptr));
+                            setWidth(atoi(values[y].value->u.object.values[z].value->u.string.ptr));
                             mask |= 1 << 4;
                         }
                         if(!strcmp(values[y].value->u.object.values[z].name, "height"))
                         {
-                            SetHeight(atoi(values[y].value->u.object.values[z].value->u.string.ptr));
+                            setHeight(atoi(values[y].value->u.object.values[z].value->u.string.ptr));
                             mask |= 1 << 5;
                         }
                     }
@@ -322,10 +346,9 @@ void JSONServer::Parse()
             }
             if(CheckMask(mask, 2))
             {
-                char *base64 = GetModel(name, format);
-                fprintf(GetOutput(),
-                        "{\n \"context\": \"%s\",\n \"model\": {\n  \"name\": \"%s\",\n  \"format\": \"%s\",\n  \"buffer\": \"%s\"\n }\n}\n",
-                        CurrentContext(), name, format, base64);
+                char *base64 = getModel(name, format);
+                fprintf(getOutput(),
+                        "{\n \"context\": \"%s\",\n \"model\": {\n  \"name\": \"%s\",\n  \"format\": \"%s\",\n  \"buffer\": \"%s\"\n }\n}\n", currentContext(), name, format, base64);
                 free(base64);
             }
         }
@@ -355,7 +378,7 @@ void JSONServer::Parse()
             }
             if(CheckMask(mask, 3))
             {
-                AddModel(name, format, base64);
+                addModel(name, format, base64);
             }
         }
         if(!strcmp(n, "dft"))
@@ -448,7 +471,7 @@ void JSONServer::Parse()
                     mask |= 1 << 6;
                 }
             }
-            if(mask & 1)
+            if(CheckMask(mask, 1))
             {
                 if(CheckMask(mask, 2)) {
                     if(!strcmp(edit, "shift"))
@@ -469,9 +492,9 @@ void JSONServer::Parse()
                         {
                             Diff(name, model, arg);
                         }
-                        else if(!strcmp(edit, "convolute"))
+                        else if(!strcmp(edit, "convolve"))
                         {
-                            Convolute(name, model, arg);
+                            Convolve(name, model, arg);
                         }
                     }
                 }
